# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501
# Custom imports
import requests
import sseclient
import json
from typing import Generator


import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictInt, StrictStr
from typing import List, Optional
from typing_extensions import Annotated
from kestrapy.models.api_autocomplete import ApiAutocomplete
from kestrapy.models.api_group_summary import ApiGroupSummary
from kestrapy.models.api_ids import ApiIds
from kestrapy.models.group_identifier_membership import GroupIdentifierMembership
from kestrapy.models.iam_group_controller_api_create_group_request import IAMGroupControllerApiCreateGroupRequest
from kestrapy.models.iam_group_controller_api_group_detail import IAMGroupControllerApiGroupDetail
from kestrapy.models.iam_group_controller_api_group_member import IAMGroupControllerApiGroupMember
from kestrapy.models.iam_group_controller_api_update_group_request import IAMGroupControllerApiUpdateGroupRequest
from kestrapy.models.paged_results_api_group_summary import PagedResultsApiGroupSummary
from kestrapy.models.paged_results_iam_group_controller_api_group_member import PagedResultsIAMGroupControllerApiGroupMember

from kestrapy.api_client import ApiClient, RequestSerialized
from kestrapy.api_response import ApiResponse
from kestrapy.rest import RESTResponseType


class GroupsApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_call
    def add_user_to_group(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the group")],
        user_id: Annotated[StrictStr, Field(description="The ID of the user to add to the group")],
        tenant: StrictStr,
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> IAMGroupControllerApiGroupMember:
        """Add a user to a group

        Adds the specified user to the given group. If the user does not already have access to the tenant, tenant access will be created automatically.

        :param id: The ID of the group (required)
        :type id: str
                :param user_id: The ID of the user to add to the group (required)
        :type user_id: str
                :param tenant: (required)
        :type tenant: str
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._add_user_to_group_serialize(
            id=id,
            user_id=user_id,
            tenant=tenant,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "IAMGroupControllerApiGroupMember",
            '404': None,
            '409': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def add_user_to_group_with_http_info(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the group")],
        user_id: Annotated[StrictStr, Field(description="The ID of the user to add to the group")],
        tenant: StrictStr,
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> ApiResponse[IAMGroupControllerApiGroupMember]:
        """Add a user to a group

        Adds the specified user to the given group. If the user does not already have access to the tenant, tenant access will be created automatically.

        :param id: The ID of the group (required)
        :type id: str
                :param user_id: The ID of the user to add to the group (required)
        :type user_id: str
                :param tenant: (required)
        :type tenant: str
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._add_user_to_group_serialize(
            id=id,
            user_id=user_id,
            tenant=tenant,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "IAMGroupControllerApiGroupMember",
            '404': None,
            '409': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    def _add_user_to_group_serialize(
        self,
        id,
        user_id,
        tenant,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        if user_id is not None:
            _path_params['userId'] = user_id
        if tenant is not None:
            _path_params['tenant'] = tenant
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter



        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'basicAuth', 
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='PUT',
            resource_path='/api/v1/{tenant}/groups/{id}/members/{userId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats
        )





    @validate_call
    def autocomplete_groups(
        self,
        tenant: StrictStr,
        api_autocomplete: Annotated[ApiAutocomplete, Field(description="Autocomplete request")],
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> List[ApiGroupSummary]:
        """List groups for autocomplete


        :param tenant: (required)
        :type tenant: str
                :param api_autocomplete: Autocomplete request (required)
        :type api_autocomplete: ApiAutocomplete
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._autocomplete_groups_serialize(
            tenant=tenant,
            api_autocomplete=api_autocomplete,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[ApiGroupSummary]",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def autocomplete_groups_with_http_info(
        self,
        tenant: StrictStr,
        api_autocomplete: Annotated[ApiAutocomplete, Field(description="Autocomplete request")],
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> ApiResponse[List[ApiGroupSummary]]:
        """List groups for autocomplete


        :param tenant: (required)
        :type tenant: str
                :param api_autocomplete: Autocomplete request (required)
        :type api_autocomplete: ApiAutocomplete
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._autocomplete_groups_serialize(
            tenant=tenant,
            api_autocomplete=api_autocomplete,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[ApiGroupSummary]",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    def _autocomplete_groups_serialize(
        self,
        tenant,
        api_autocomplete,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if tenant is not None:
            _path_params['tenant'] = tenant
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if api_autocomplete is not None:
            _body_params = api_autocomplete



        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`

        _default_content_type = (
            self.api_client.select_header_content_type(
                [
                    'application/json'
                ]
            )
        )
        if _default_content_type is not None:
            _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'basicAuth', 
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/api/v1/{tenant}/groups/autocomplete',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats
        )





    @validate_call
    def create_group(
        self,
        tenant: StrictStr,
        iam_group_controller_api_create_group_request: Annotated[IAMGroupControllerApiCreateGroupRequest, Field(description="The group")],
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> IAMGroupControllerApiGroupDetail:
        """Create a group


        :param tenant: (required)
        :type tenant: str
                :param iam_group_controller_api_create_group_request: The group (required)
        :type iam_group_controller_api_create_group_request: IAMGroupControllerApiCreateGroupRequest
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._create_group_serialize(
            tenant=tenant,
            iam_group_controller_api_create_group_request=iam_group_controller_api_create_group_request,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "IAMGroupControllerApiGroupDetail",
            '409': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def create_group_with_http_info(
        self,
        tenant: StrictStr,
        iam_group_controller_api_create_group_request: Annotated[IAMGroupControllerApiCreateGroupRequest, Field(description="The group")],
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> ApiResponse[IAMGroupControllerApiGroupDetail]:
        """Create a group


        :param tenant: (required)
        :type tenant: str
                :param iam_group_controller_api_create_group_request: The group (required)
        :type iam_group_controller_api_create_group_request: IAMGroupControllerApiCreateGroupRequest
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._create_group_serialize(
            tenant=tenant,
            iam_group_controller_api_create_group_request=iam_group_controller_api_create_group_request,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "IAMGroupControllerApiGroupDetail",
            '409': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    def _create_group_serialize(
        self,
        tenant,
        iam_group_controller_api_create_group_request,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if tenant is not None:
            _path_params['tenant'] = tenant
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if iam_group_controller_api_create_group_request is not None:
            _body_params = iam_group_controller_api_create_group_request



        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`

        _default_content_type = (
            self.api_client.select_header_content_type(
                [
                    'application/json'
                ]
            )
        )
        if _default_content_type is not None:
            _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'basicAuth', 
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/api/v1/{tenant}/groups',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats
        )





    @validate_call
    def delete_group(
        self,
        id: Annotated[StrictStr, Field(description="The group id")],
        tenant: StrictStr,
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> None:
        """Delete a group


        :param id: The group id (required)
        :type id: str
                :param tenant: (required)
        :type tenant: str
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._delete_group_serialize(
            id=id,
            tenant=tenant,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
            '404': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def delete_group_with_http_info(
        self,
        id: Annotated[StrictStr, Field(description="The group id")],
        tenant: StrictStr,
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> ApiResponse[None]:
        """Delete a group


        :param id: The group id (required)
        :type id: str
                :param tenant: (required)
        :type tenant: str
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._delete_group_serialize(
            id=id,
            tenant=tenant,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
            '404': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    def _delete_group_serialize(
        self,
        id,
        tenant,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        if tenant is not None:
            _path_params['tenant'] = tenant
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter





        # authentication setting
        _auth_settings: List[str] = [
            'basicAuth', 
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/api/v1/{tenant}/groups/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats
        )





    @validate_call
    def delete_user_from_group(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the group")],
        user_id: Annotated[StrictStr, Field(description="The ID of the user to remove from the group")],
        tenant: StrictStr,
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> IAMGroupControllerApiGroupMember:
        """Remove a user from a group

        Removes the specified user from the given group. If the user has no other group bindings within the tenant, their access to the tenant will also be revoked.

        :param id: The ID of the group (required)
        :type id: str
                :param user_id: The ID of the user to remove from the group (required)
        :type user_id: str
                :param tenant: (required)
        :type tenant: str
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._delete_user_from_group_serialize(
            id=id,
            user_id=user_id,
            tenant=tenant,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "IAMGroupControllerApiGroupMember",
            '404': None,
            '409': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def delete_user_from_group_with_http_info(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the group")],
        user_id: Annotated[StrictStr, Field(description="The ID of the user to remove from the group")],
        tenant: StrictStr,
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> ApiResponse[IAMGroupControllerApiGroupMember]:
        """Remove a user from a group

        Removes the specified user from the given group. If the user has no other group bindings within the tenant, their access to the tenant will also be revoked.

        :param id: The ID of the group (required)
        :type id: str
                :param user_id: The ID of the user to remove from the group (required)
        :type user_id: str
                :param tenant: (required)
        :type tenant: str
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._delete_user_from_group_serialize(
            id=id,
            user_id=user_id,
            tenant=tenant,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "IAMGroupControllerApiGroupMember",
            '404': None,
            '409': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    def _delete_user_from_group_serialize(
        self,
        id,
        user_id,
        tenant,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        if user_id is not None:
            _path_params['userId'] = user_id
        if tenant is not None:
            _path_params['tenant'] = tenant
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter



        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'basicAuth', 
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/api/v1/{tenant}/groups/{id}/members/{userId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats
        )





    @validate_call
    def group(
        self,
        id: Annotated[StrictStr, Field(description="The group id")],
        tenant: StrictStr,
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> IAMGroupControllerApiGroupDetail:
        """Retrieve a group

        Retrieves details of a specific group by its ID within the current tenant.

        :param id: The group id (required)
        :type id: str
                :param tenant: (required)
        :type tenant: str
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._group_serialize(
            id=id,
            tenant=tenant,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "IAMGroupControllerApiGroupDetail",
            '404': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def group_with_http_info(
        self,
        id: Annotated[StrictStr, Field(description="The group id")],
        tenant: StrictStr,
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> ApiResponse[IAMGroupControllerApiGroupDetail]:
        """Retrieve a group

        Retrieves details of a specific group by its ID within the current tenant.

        :param id: The group id (required)
        :type id: str
                :param tenant: (required)
        :type tenant: str
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._group_serialize(
            id=id,
            tenant=tenant,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "IAMGroupControllerApiGroupDetail",
            '404': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    def _group_serialize(
        self,
        id,
        tenant,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        if tenant is not None:
            _path_params['tenant'] = tenant
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter



        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'basicAuth', 
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/v1/{tenant}/groups/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats
        )





    @validate_call
    def list_group_ids(
        self,
        tenant: StrictStr,
        api_ids: Annotated[ApiIds, Field(description="The ids that must be present on results")],
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> List[ApiGroupSummary]:
        """List groups by ids


        :param tenant: (required)
        :type tenant: str
                :param api_ids: The ids that must be present on results (required)
        :type api_ids: ApiIds
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._list_group_ids_serialize(
            tenant=tenant,
            api_ids=api_ids,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[ApiGroupSummary]",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def list_group_ids_with_http_info(
        self,
        tenant: StrictStr,
        api_ids: Annotated[ApiIds, Field(description="The ids that must be present on results")],
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> ApiResponse[List[ApiGroupSummary]]:
        """List groups by ids


        :param tenant: (required)
        :type tenant: str
                :param api_ids: The ids that must be present on results (required)
        :type api_ids: ApiIds
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._list_group_ids_serialize(
            tenant=tenant,
            api_ids=api_ids,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[ApiGroupSummary]",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    def _list_group_ids_serialize(
        self,
        tenant,
        api_ids,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if tenant is not None:
            _path_params['tenant'] = tenant
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if api_ids is not None:
            _body_params = api_ids



        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`

        _default_content_type = (
            self.api_client.select_header_content_type(
                [
                    'application/json'
                ]
            )
        )
        if _default_content_type is not None:
            _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'basicAuth', 
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/api/v1/{tenant}/groups/ids',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats
        )





    @validate_call
    def search_group_members(
        self,
        id: Annotated[StrictStr, Field(description="The group id")],
        page: Annotated[StrictInt, Field(description="The current page")],
        size: Annotated[StrictInt, Field(description="The current page size")],
        tenant: StrictStr,
        q: Annotated[Optional[StrictStr], Field(description="A string filter")] = None,
        sort: Annotated[Optional[List[StrictStr]], Field(description="The sort of current page")] = None,
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> PagedResultsIAMGroupControllerApiGroupMember:
        """Search for users in a group


        :param id: The group id (required)
        :type id: str
                :param page: The current page (required)
        :type page: int
                :param size: The current page size (required)
        :type size: int
                :param tenant: (required)
        :type tenant: str
                :param q: A string filter
        :type q: str
                :param sort: The sort of current page
        :type sort: List[str]
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._search_group_members_serialize(
            id=id,
            page=page,
            size=size,
            tenant=tenant,
            q=q,
            sort=sort,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PagedResultsIAMGroupControllerApiGroupMember",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def search_group_members_with_http_info(
        self,
        id: Annotated[StrictStr, Field(description="The group id")],
        page: Annotated[StrictInt, Field(description="The current page")],
        size: Annotated[StrictInt, Field(description="The current page size")],
        tenant: StrictStr,
        q: Annotated[Optional[StrictStr], Field(description="A string filter")] = None,
        sort: Annotated[Optional[List[StrictStr]], Field(description="The sort of current page")] = None,
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> ApiResponse[PagedResultsIAMGroupControllerApiGroupMember]:
        """Search for users in a group


        :param id: The group id (required)
        :type id: str
                :param page: The current page (required)
        :type page: int
                :param size: The current page size (required)
        :type size: int
                :param tenant: (required)
        :type tenant: str
                :param q: A string filter
        :type q: str
                :param sort: The sort of current page
        :type sort: List[str]
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._search_group_members_serialize(
            id=id,
            page=page,
            size=size,
            tenant=tenant,
            q=q,
            sort=sort,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PagedResultsIAMGroupControllerApiGroupMember",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    def _search_group_members_serialize(
        self,
        id,
        page,
        size,
        tenant,
        q,
        sort,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'sort': 'csv',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        if tenant is not None:
            _path_params['tenant'] = tenant
        # process the query parameters
        if q is not None:
            
            _query_params.append(('q', q))
            
        if page is not None:
            
            _query_params.append(('page', page))
            
        if size is not None:
            
            _query_params.append(('size', size))
            
        if sort is not None:
            
            _query_params.append(('sort', sort))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter



        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'basicAuth', 
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/v1/{tenant}/groups/{id}/members',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats
        )





    @validate_call
    def search_groups(
        self,
        page: Annotated[StrictInt, Field(description="The current page")],
        size: Annotated[StrictInt, Field(description="The current page size")],
        tenant: StrictStr,
        q: Annotated[Optional[StrictStr], Field(description="A string filter")] = None,
        sort: Annotated[Optional[List[StrictStr]], Field(description="The sort of current page")] = None,
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> PagedResultsApiGroupSummary:
        """Search for groups


        :param page: The current page (required)
        :type page: int
                :param size: The current page size (required)
        :type size: int
                :param tenant: (required)
        :type tenant: str
                :param q: A string filter
        :type q: str
                :param sort: The sort of current page
        :type sort: List[str]
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._search_groups_serialize(
            page=page,
            size=size,
            tenant=tenant,
            q=q,
            sort=sort,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PagedResultsApiGroupSummary",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def search_groups_with_http_info(
        self,
        page: Annotated[StrictInt, Field(description="The current page")],
        size: Annotated[StrictInt, Field(description="The current page size")],
        tenant: StrictStr,
        q: Annotated[Optional[StrictStr], Field(description="A string filter")] = None,
        sort: Annotated[Optional[List[StrictStr]], Field(description="The sort of current page")] = None,
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> ApiResponse[PagedResultsApiGroupSummary]:
        """Search for groups


        :param page: The current page (required)
        :type page: int
                :param size: The current page size (required)
        :type size: int
                :param tenant: (required)
        :type tenant: str
                :param q: A string filter
        :type q: str
                :param sort: The sort of current page
        :type sort: List[str]
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._search_groups_serialize(
            page=page,
            size=size,
            tenant=tenant,
            q=q,
            sort=sort,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PagedResultsApiGroupSummary",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    def _search_groups_serialize(
        self,
        page,
        size,
        tenant,
        q,
        sort,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'sort': 'csv',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if tenant is not None:
            _path_params['tenant'] = tenant
        # process the query parameters
        if q is not None:
            
            _query_params.append(('q', q))
            
        if page is not None:
            
            _query_params.append(('page', page))
            
        if size is not None:
            
            _query_params.append(('size', size))
            
        if sort is not None:
            
            _query_params.append(('sort', sort))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter



        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'basicAuth', 
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/v1/{tenant}/groups/search',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats
        )





    @validate_call
    def set_user_membership_for_group(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the group")],
        user_id: Annotated[StrictStr, Field(description="The ID of the user whose membership is being updated")],
        membership: Annotated[GroupIdentifierMembership, Field(description="The new membership type to assign to the user.")],
        tenant: StrictStr,
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> IAMGroupControllerApiGroupMember:
        """Update a user's membership type in a group

        Allows a group owner or an authorized user to change the role of a user within a group to OWNER or MEMBER.

        :param id: The ID of the group (required)
        :type id: str
                :param user_id: The ID of the user whose membership is being updated (required)
        :type user_id: str
                :param membership: The new membership type to assign to the user. (required)
        :type membership: GroupIdentifierMembership
                :param tenant: (required)
        :type tenant: str
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._set_user_membership_for_group_serialize(
            id=id,
            user_id=user_id,
            membership=membership,
            tenant=tenant,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "IAMGroupControllerApiGroupMember",
            '404': None,
            '409': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def set_user_membership_for_group_with_http_info(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the group")],
        user_id: Annotated[StrictStr, Field(description="The ID of the user whose membership is being updated")],
        membership: Annotated[GroupIdentifierMembership, Field(description="The new membership type to assign to the user.")],
        tenant: StrictStr,
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> ApiResponse[IAMGroupControllerApiGroupMember]:
        """Update a user's membership type in a group

        Allows a group owner or an authorized user to change the role of a user within a group to OWNER or MEMBER.

        :param id: The ID of the group (required)
        :type id: str
                :param user_id: The ID of the user whose membership is being updated (required)
        :type user_id: str
                :param membership: The new membership type to assign to the user. (required)
        :type membership: GroupIdentifierMembership
                :param tenant: (required)
        :type tenant: str
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._set_user_membership_for_group_serialize(
            id=id,
            user_id=user_id,
            membership=membership,
            tenant=tenant,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "IAMGroupControllerApiGroupMember",
            '404': None,
            '409': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    def _set_user_membership_for_group_serialize(
        self,
        id,
        user_id,
        membership,
        tenant,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        if user_id is not None:
            _path_params['userId'] = user_id
        if tenant is not None:
            _path_params['tenant'] = tenant
        # process the query parameters
        if membership is not None:
            
            _query_params.append(('membership', membership.value))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter



        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'basicAuth', 
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='PUT',
            resource_path='/api/v1/{tenant}/groups/{id}/members/membership/{userId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats
        )





    @validate_call
    def update_group(
        self,
        id: Annotated[StrictStr, Field(description="The group id")],
        tenant: StrictStr,
        iam_group_controller_api_update_group_request: Annotated[IAMGroupControllerApiUpdateGroupRequest, Field(description="The group")],
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> IAMGroupControllerApiGroupDetail:
        """Update a group


        :param id: The group id (required)
        :type id: str
                :param tenant: (required)
        :type tenant: str
                :param iam_group_controller_api_update_group_request: The group (required)
        :type iam_group_controller_api_update_group_request: IAMGroupControllerApiUpdateGroupRequest
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._update_group_serialize(
            id=id,
            tenant=tenant,
            iam_group_controller_api_update_group_request=iam_group_controller_api_update_group_request,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "IAMGroupControllerApiGroupDetail",
            '404': None,
            '409': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def update_group_with_http_info(
        self,
        id: Annotated[StrictStr, Field(description="The group id")],
        tenant: StrictStr,
        iam_group_controller_api_update_group_request: Annotated[IAMGroupControllerApiUpdateGroupRequest, Field(description="The group")],
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> ApiResponse[IAMGroupControllerApiGroupDetail]:
        """Update a group


        :param id: The group id (required)
        :type id: str
                :param tenant: (required)
        :type tenant: str
                :param iam_group_controller_api_update_group_request: The group (required)
        :type iam_group_controller_api_update_group_request: IAMGroupControllerApiUpdateGroupRequest
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._update_group_serialize(
            id=id,
            tenant=tenant,
            iam_group_controller_api_update_group_request=iam_group_controller_api_update_group_request,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "IAMGroupControllerApiGroupDetail",
            '404': None,
            '409': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    def _update_group_serialize(
        self,
        id,
        tenant,
        iam_group_controller_api_update_group_request,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        if tenant is not None:
            _path_params['tenant'] = tenant
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if iam_group_controller_api_update_group_request is not None:
            _body_params = iam_group_controller_api_update_group_request



        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`

        _default_content_type = (
            self.api_client.select_header_content_type(
                [
                    'application/json'
                ]
            )
        )
        if _default_content_type is not None:
            _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'basicAuth', 
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='PUT',
            resource_path='/api/v1/{tenant}/groups/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats
        )







from .api_client import ApiClient
from .api.flows_api import FlowsApi
from .api.executions_api import ExecutionsApi
from .api.groups_api import GroupsApi
from .api.kv_api import KVApi
from .api.namespaces_api import NamespacesApi
from .api.roles_api import RolesApi
from .api.service_account_api import ServiceAccountApi
from .api.triggers_api import TriggersApi
from .api.users_api import UsersApi


class KestraClient:
    flows: FlowsApi = None
    executions: ExecutionsApi = None
    groups: GroupsApi = None
    kv: KVApi = None
    namespaces: NamespacesApi = None
    roles: RolesApi = None
    serviceAccount: ServiceAccountApi = None
    triggers: TriggersApi = None
    users: UsersApi = None

    def __init__(self, configuration=None):
        if configuration is None:
            configuration = ApiClient().configuration
        self.api_client = ApiClient(configuration=configuration)

        self.flows = FlowsApi(self.api_client)
        self.executions = ExecutionsApi(self.api_client)
        self.groups = GroupsApi(self.api_client)
        self.kv = KVApi(self.api_client)
        self.namespaces = NamespacesApi(self.api_client)
        self.roles = RolesApi(self.api_client)
        self.service_account = ServiceAccountApi(self.api_client)
        self.triggers = TriggersApi(self.api_client)
        self.users = UsersApi(self.api_client)



# flake8: noqa

# import apis into api package
from kestrapy.api.executions_api import ExecutionsApi
from kestrapy.api.flows_api import FlowsApi
from kestrapy.api.groups_api import GroupsApi
from kestrapy.api.kv_api import KVApi
from kestrapy.api.namespaces_api import NamespacesApi
from kestrapy.api.roles_api import RolesApi
from kestrapy.api.service_account_api import ServiceAccountApi
from kestrapy.api.triggers_api import TriggersApi
from kestrapy.api.users_api import UsersApi




# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import regex as re
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from kestrapy.models.breakpoint import Breakpoint
from kestrapy.models.execution_kind import ExecutionKind
from kestrapy.models.execution_metadata import ExecutionMetadata
from kestrapy.models.execution_trigger import ExecutionTrigger
from kestrapy.models.label import Label
from kestrapy.models.state import State
from kestrapy.models.task_fixture import TaskFixture
from kestrapy.models.task_run import TaskRun
from typing import Optional, Set
from typing_extensions import Self

class ExecutionControllerExecutionResponse(BaseModel):
    """
    ExecutionControllerExecutionResponse
    """ # noqa: E501
    labels: Optional[List[Label]] = None
    id: StrictStr
    namespace: StrictStr
    flow_id: StrictStr = Field(alias="flowId")
    flow_revision: StrictInt = Field(alias="flowRevision")
    task_run_list: Optional[List[TaskRun]] = Field(default=None, alias="taskRunList")
    inputs: Optional[Dict[str, Any]] = None
    outputs: Optional[Dict[str, Any]] = None
    variables: Optional[Dict[str, Any]] = None
    state: State
    parent_id: Optional[StrictStr] = Field(default=None, alias="parentId")
    original_id: Optional[StrictStr] = Field(default=None, alias="originalId")
    trigger: Optional[ExecutionTrigger] = None
    deleted: StrictBool
    metadata: Optional[ExecutionMetadata] = None
    schedule_date: Optional[datetime] = Field(default=None, alias="scheduleDate")
    trace_parent: Optional[StrictStr] = Field(default=None, alias="traceParent")
    fixtures: Optional[List[TaskFixture]] = None
    kind: Optional[ExecutionKind] = None
    breakpoints: Optional[List[Breakpoint]] = None
    url: Optional[StrictStr] = None
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["labels", "id", "namespace", "flowId", "flowRevision", "taskRunList", "inputs", "outputs", "variables", "state", "parentId", "originalId", "trigger", "deleted", "metadata", "scheduleDate", "traceParent", "fixtures", "kind", "breakpoints", "url"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ExecutionControllerExecutionResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in labels (list)
        _items = []
        if self.labels:
            for _item_labels in self.labels:
                if _item_labels:
                    _items.append(_item_labels.to_dict())
            _dict['labels'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in task_run_list (list)
        _items = []
        if self.task_run_list:
            for _item_task_run_list in self.task_run_list:
                if _item_task_run_list:
                    _items.append(_item_task_run_list.to_dict())
            _dict['taskRunList'] = _items
        # override the default output from pydantic by calling `to_dict()` of state
        if self.state:
            _dict['state'] = self.state.to_dict()
        # override the default output from pydantic by calling `to_dict()` of trigger
        if self.trigger:
            _dict['trigger'] = self.trigger.to_dict()
        # override the default output from pydantic by calling `to_dict()` of metadata
        if self.metadata:
            _dict['metadata'] = self.metadata.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in fixtures (list)
        _items = []
        if self.fixtures:
            for _item_fixtures in self.fixtures:
                if _item_fixtures:
                    _items.append(_item_fixtures.to_dict())
            _dict['fixtures'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in breakpoints (list)
        _items = []
        if self.breakpoints:
            for _item_breakpoints in self.breakpoints:
                if _item_breakpoints:
                    _items.append(_item_breakpoints.to_dict())
            _dict['breakpoints'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        # set to None if schedule_date (nullable) is None
        # and model_fields_set contains the field
        if self.schedule_date is None and "schedule_date" in self.model_fields_set:
            _dict['scheduleDate'] = None

        # set to None if fixtures (nullable) is None
        # and model_fields_set contains the field
        if self.fixtures is None and "fixtures" in self.model_fields_set:
            _dict['fixtures'] = None

        # set to None if kind (nullable) is None
        # and model_fields_set contains the field
        if self.kind is None and "kind" in self.model_fields_set:
            _dict['kind'] = None

        # set to None if breakpoints (nullable) is None
        # and model_fields_set contains the field
        if self.breakpoints is None and "breakpoints" in self.model_fields_set:
            _dict['breakpoints'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ExecutionControllerExecutionResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "labels": [Label.from_dict(_item) for _item in obj["labels"]] if obj.get("labels") is not None else None,
            "id": obj.get("id"),
            "namespace": obj.get("namespace"),
            "flowId": obj.get("flowId"),
            "flowRevision": obj.get("flowRevision"),
            "taskRunList": [TaskRun.from_dict(_item) for _item in obj["taskRunList"]] if obj.get("taskRunList") is not None else None,
            "inputs": obj.get("inputs"),
            "outputs": obj.get("outputs"),
            "variables": obj.get("variables"),
            "state": State.from_dict(obj["state"]) if obj.get("state") is not None else None,
            "parentId": obj.get("parentId"),
            "originalId": obj.get("originalId"),
            "trigger": ExecutionTrigger.from_dict(obj["trigger"]) if obj.get("trigger") is not None else None,
            "deleted": obj.get("deleted"),
            "metadata": ExecutionMetadata.from_dict(obj["metadata"]) if obj.get("metadata") is not None else None,
            "scheduleDate": obj.get("scheduleDate"),
            "traceParent": obj.get("traceParent"),
            "fixtures": [TaskFixture.from_dict(_item) for _item in obj["fixtures"]] if obj.get("fixtures") is not None else None,
            "kind": obj.get("kind"),
            "breakpoints": [Breakpoint.from_dict(_item) for _item in obj["breakpoints"]] if obj.get("breakpoints") is not None else None,
            "url": obj.get("url")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj





# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
from kestrapy import Configuration, KestraClient
from kestrapy import (IAMServiceAccountControllerApiCreateServiceAccountRequest,
                      IAMServiceAccountControllerApiServiceAccountRequest,
                      IAMServiceAccountControllerApiPatchServiceAccountRequest,
                      ApiPatchSuperAdminRequest)


class TestServiceAccountApi(unittest.TestCase):
    """ServiceAccountApi unit test stubs"""

    def setUp(self) -> None:
        configuration = Configuration()
        configuration.host = "http://localhost:9902"
        configuration.username = "root@root.com"
        configuration.password = "Root!1234"

        self.kestra_client = KestraClient(configuration)
        self.tenant = "main"

    def tearDown(self) -> None:
        pass

    def test_create_service_account(self) -> None:
        """Test case for create_service_account

        Create a service account
        """
        name = "test-create-service-account"
        req = IAMServiceAccountControllerApiCreateServiceAccountRequest(
            name=f"{name}",
            description="service account created by tests"
        )

        created = self.kestra_client.service_account.create_service_account(
            iam_service_account_controller_api_create_service_account_request=req
        )
        assert getattr(created, 'id', None) is not None

    def test_create_service_account_for_tenant(self) -> None:
        """Test case for create_service_account_for_tenant

        Create a service account for the given tenant
        """
        name = "test-create-service-account-for-tenant"
        req = IAMServiceAccountControllerApiServiceAccountRequest(
            name=f"{name}",
            description="service account for tenant"
        )

        created = self.kestra_client.service_account.create_service_account_for_tenant(
            tenant=self.tenant,
            iam_service_account_controller_api_service_account_request=req
        )
        assert getattr(created, 'id', None) is not None

    def test_delete_service_account(self) -> None:
        """Test case for delete_service_account

        Delete a service account
        """
        name = "test-delete-service-account"
        req = IAMServiceAccountControllerApiCreateServiceAccountRequest(name=f"{name}")
        created = self.kestra_client.service_account.create_service_account(iam_service_account_controller_api_create_service_account_request=req)

        self.kestra_client.service_account.delete_service_account(id=created.id)

    def test_delete_service_account_for_tenant(self) -> None:
        """Test case for delete_service_account_for_tenant

        Delete a service account
        """
        name = "test-delete-service-account-for-tenant"
        req = IAMServiceAccountControllerApiServiceAccountRequest(name=f"{name}")
        created = self.kestra_client.service_account.create_service_account_for_tenant(tenant=self.tenant, iam_service_account_controller_api_service_account_request=req)

        self.kestra_client.service_account.delete_service_account_for_tenant(tenant=self.tenant, id=created.id)

    def test_get_service_account(self) -> None:
        """Test case for get_service_account

        Get a service account
        """
        name = "test-get-service-account"
        req = IAMServiceAccountControllerApiCreateServiceAccountRequest(name=f"{name}")
        created = self.kestra_client.service_account.create_service_account(iam_service_account_controller_api_create_service_account_request=req)

        fetched = self.kestra_client.service_account.service_account(id=created.id)
        assert getattr(fetched, 'id', None) == created.id

    def test_get_service_account_for_tenant(self) -> None:
        """Test case for get_service_account_for_tenant

        Retrieve a service account
        """
        name = "test-get-service-account-for-tenant"
        req = IAMServiceAccountControllerApiServiceAccountRequest(name=f"{name}")
        created = self.kestra_client.service_account.create_service_account_for_tenant(tenant=self.tenant, iam_service_account_controller_api_service_account_request=req)

        fetched = self.kestra_client.service_account.service_account_for_tenant(tenant=self.tenant, id=created.id)
        assert getattr(fetched, 'id', None) == created.id

    def test_list_service_accounts(self) -> None:
        """Test case for list_service_accounts

        List service accounts. Superadmin-only. 
        """
        name = "test-list-service-accounts"
        req = IAMServiceAccountControllerApiCreateServiceAccountRequest(name=f"{name}")
        created = self.kestra_client.service_account.create_service_account(iam_service_account_controller_api_create_service_account_request=req)

        results = self.kestra_client.service_account.list_service_accounts(page=1, size=50)
        # results may be a dict or object - try both
        ids = None
        if isinstance(results, dict):
            ids = [r.get('id') for r in results.get('results', [])]
        else:
            ids = [getattr(r, 'id', None) for r in getattr(results, 'results', [])]

        assert created.id in ids


    def test_patch_service_account_details(self) -> None:
        """Test case for patch_service_account_details

        Update service account details
        """
        name = "test-patch-service-account-details"
        req = IAMServiceAccountControllerApiCreateServiceAccountRequest(name=f"{name}", description="old")
        created = self.kestra_client.service_account.create_service_account(iam_service_account_controller_api_create_service_account_request=req)

        patch_req = IAMServiceAccountControllerApiPatchServiceAccountRequest(name=f"{name}", description="new")
        patched = self.kestra_client.service_account.patch_service_account_details(id=created.id, iam_service_account_controller_api_patch_service_account_request=patch_req)
        assert getattr(patched, 'description', None) == "new" or getattr(patched, 'desc', None) == "new"

    def test_patch_service_account_super_admin(self) -> None:
        """Test case for patch_service_account_super_admin

        Update service account superadmin privileges
        """
        name = "test-patch-service-account-super-admin"
        req = IAMServiceAccountControllerApiCreateServiceAccountRequest(name=f"{name}")
        created = self.kestra_client.service_account.create_service_account(iam_service_account_controller_api_create_service_account_request=req)

        patch = ApiPatchSuperAdminRequest.from_dict({"superAdmin": True})
        self.kestra_client.service_account.patch_service_account_super_admin(id=created.id, api_patch_super_admin_request=patch)

        fetched = self.kestra_client.service_account.service_account(id=created.id)
        assert getattr(fetched, 'super_admin', None) is True

    def test_update_service_account(self) -> None:
        """Test case for update_service_account

        Update a user service account
        """
        name = "test-update-service-account"
        req = IAMServiceAccountControllerApiCreateServiceAccountRequest(name=f"{name}", description="Before")
        created = self.kestra_client.service_account.create_service_account(iam_service_account_controller_api_create_service_account_request=req)

        update_req = IAMServiceAccountControllerApiServiceAccountRequest(name=created.name, description="After")
        # update_service_account requires the tenant argument and the 'service_account_request' parameter name
        updated = self.kestra_client.service_account.update_service_account(id=created.id, tenant=self.tenant, iam_service_account_controller_api_service_account_request=update_req)
        assert getattr(updated, 'description', None) == "After" or getattr(updated, 'desc', None) == "After"

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import regex as re
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from kestrapy.models.concurrency import Concurrency
from kestrapy.models.input_object import InputObject
from kestrapy.models.label import Label
from kestrapy.models.output import Output
from kestrapy.models.sla import SLA
from kestrapy.models.worker_group import WorkerGroup
from typing import Optional, Set
from typing_extensions import Self

class FlowInterface(BaseModel):
    """
    FlowInterface
    """ # noqa: E501
    id: Optional[StrictStr] = None
    namespace: Optional[StrictStr] = None
    revision: Optional[StrictInt] = None
    tenant_id: Optional[StrictStr] = Field(default=None, alias="tenantId")
    deleted: Optional[StrictBool] = None
    description: Optional[StrictStr] = None
    disabled: Optional[StrictBool] = None
    labels: Optional[List[Label]] = None
    inputs: Optional[List[InputObject]] = None
    outputs: Optional[List[Output]] = None
    variables: Optional[Dict[str, Dict[str, Any]]] = None
    worker_group: Optional[WorkerGroup] = Field(default=None, alias="workerGroup")
    concurrency: Optional[Concurrency] = None
    sla: Optional[List[SLA]] = None
    source: Optional[StrictStr] = None
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["id", "namespace", "revision", "tenantId", "deleted", "description", "disabled", "labels", "inputs", "outputs", "variables", "workerGroup", "concurrency", "sla", "source"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of FlowInterface from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in labels (list)
        _items = []
        if self.labels:
            for _item_labels in self.labels:
                if _item_labels:
                    _items.append(_item_labels.to_dict())
            _dict['labels'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in inputs (list)
        _items = []
        if self.inputs:
            for _item_inputs in self.inputs:
                if _item_inputs:
                    _items.append(_item_inputs.to_dict())
            _dict['inputs'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in outputs (list)
        _items = []
        if self.outputs:
            for _item_outputs in self.outputs:
                if _item_outputs:
                    _items.append(_item_outputs.to_dict())
            _dict['outputs'] = _items
        # override the default output from pydantic by calling `to_dict()` of worker_group
        if self.worker_group:
            _dict['workerGroup'] = self.worker_group.to_dict()
        # override the default output from pydantic by calling `to_dict()` of concurrency
        if self.concurrency:
            _dict['concurrency'] = self.concurrency.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in sla (list)
        _items = []
        if self.sla:
            for _item_sla in self.sla:
                if _item_sla:
                    _items.append(_item_sla.to_dict())
            _dict['sla'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of FlowInterface from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "namespace": obj.get("namespace"),
            "revision": obj.get("revision"),
            "tenantId": obj.get("tenantId"),
            "deleted": obj.get("deleted"),
            "description": obj.get("description"),
            "disabled": obj.get("disabled"),
            "labels": [Label.from_dict(_item) for _item in obj["labels"]] if obj.get("labels") is not None else None,
            "inputs": [InputObject.from_dict(_item) for _item in obj["inputs"]] if obj.get("inputs") is not None else None,
            "outputs": [Output.from_dict(_item) for _item in obj["outputs"]] if obj.get("outputs") is not None else None,
            "variables": obj.get("variables"),
            "workerGroup": WorkerGroup.from_dict(obj["workerGroup"]) if obj.get("workerGroup") is not None else None,
            "concurrency": Concurrency.from_dict(obj["concurrency"]) if obj.get("concurrency") is not None else None,
            "sla": [SLA.from_dict(_item) for _item in obj["sla"]] if obj.get("sla") is not None else None,
            "source": obj.get("source")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj





# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    The version of the OpenAPI document: 1.1.4
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from kestrapy.api.kv_api import KVApi


class TestKVApi(unittest.TestCase):
    """KVApi unit test stubs"""

    def setUp(self) -> None:
        self.api = KVApi()

    def tearDown(self) -> None:
        pass

    def test_delete_key_value(self) -> None:
        """Test case for delete_key_value

        Delete a key-value pair
        """
        pass

    def test_delete_key_values(self) -> None:
        """Test case for delete_key_values

        Bulk-delete multiple key/value pairs from the given namespace.
        """
        pass

    def test_key_value(self) -> None:
        """Test case for key_value

        Get value for a key
        """
        pass

    def test_list_all_keys(self) -> None:
        """Test case for list_all_keys

        List all keys
        """
        pass

    def test_list_keys_with_inheritence(self) -> None:
        """Test case for list_keys_with_inheritence

        List all keys for inherited namespaces
        """
        pass

    def test_set_key_value(self) -> None:
        """Test case for set_key_value

        Puts a key-value pair in store
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from setuptools import setup, find_packages  # noqa: H301

# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools
NAME = "@kestra-io/kestrapy"
VERSION = "1.0.7"
PYTHON_REQUIRES = ">= 3.9"
REQUIRES = [
"requests (>= 2.32.5)",
"sseclient-py (>= 1.8.0)",
"urllib3 >= 2.1.0, < 3.0.0",
"python-dateutil >= 2.8.2",
"pydantic >= 2",
"typing-extensions >= 4.7.1",
]

setup(
name=NAME,
version=VERSION,
description="Kestra EE",
author="OpenAPI Generator community",
author_email="team@openapitools.org",
url="",
keywords=["OpenAPI", "OpenAPI-Generator", "Kestra EE"],
install_requires=REQUIRES,
packages=find_packages(exclude=["test", "tests"]),
include_package_data=True,
long_description_content_type='text/markdown',
long_description="""\
All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.&lt;br/&gt; Endpoints designated as Superadmin-only are not tenant-scoped.
""",  # noqa: E501
package_data={"kestrapy": ["py.typed"]},
)


# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import regex as re
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from kestrapy.models.binding import Binding
from kestrapy.models.invitation_invitation_status import InvitationInvitationStatus
from kestrapy.models.user_type import UserType
from typing import Optional, Set
from typing_extensions import Self

class Invitation(BaseModel):
    """
    Invitation
    """ # noqa: E501
    is_expired: Optional[StrictBool] = Field(default=None, alias="isExpired")
    email: Annotated[str, Field(strict=True)]
    id: Optional[StrictStr] = None
    bindings: Optional[List[Binding]] = None
    group_ids: Optional[List[StrictStr]] = Field(default=None, alias="groupIds")
    tenant_id: Optional[StrictStr] = Field(default=None, alias="tenantId")
    status: Optional[InvitationInvitationStatus] = None
    sent_at: Optional[datetime] = Field(default=None, alias="sentAt")
    expired_at: Optional[datetime] = Field(default=None, alias="expiredAt")
    accepted_at: Optional[datetime] = Field(default=None, alias="acceptedAt")
    deleted: StrictBool
    user_type: Optional[UserType] = Field(default=None, alias="userType")
    super_admin: Optional[StrictBool] = Field(default=None, alias="superAdmin")
    link: Optional[StrictStr] = None
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["isExpired", "email", "id", "bindings", "groupIds", "tenantId", "status", "sentAt", "expiredAt", "acceptedAt", "deleted", "userType", "superAdmin", "link"]

    @field_validator('email')
    def email_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^$|^[a-zA-Z0-9_!#$%&’*+\/=?`{|}~^.-]+@[a-zA-Z0-9.-]+$", value):
            raise ValueError(r"must validate the regular expression /^$|^[a-zA-Z0-9_!#$%&’*+\/=?`{|}~^.-]+@[a-zA-Z0-9.-]+$/")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Invitation from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in bindings (list)
        _items = []
        if self.bindings:
            for _item_bindings in self.bindings:
                if _item_bindings:
                    _items.append(_item_bindings.to_dict())
            _dict['bindings'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Invitation from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "isExpired": obj.get("isExpired"),
            "email": obj.get("email"),
            "id": obj.get("id"),
            "bindings": [Binding.from_dict(_item) for _item in obj["bindings"]] if obj.get("bindings") is not None else None,
            "groupIds": obj.get("groupIds"),
            "tenantId": obj.get("tenantId"),
            "status": obj.get("status"),
            "sentAt": obj.get("sentAt"),
            "expiredAt": obj.get("expiredAt"),
            "acceptedAt": obj.get("acceptedAt"),
            "deleted": obj.get("deleted"),
            "userType": obj.get("userType"),
            "superAdmin": obj.get("superAdmin"),
            "link": obj.get("link")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj





# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    The version of the OpenAPI document: 1.1.4
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from kestrapy.api.executions_api import ExecutionsApi


class TestExecutionsApi(unittest.TestCase):
    """ExecutionsApi unit test stubs"""

    def setUp(self) -> None:
        self.api = ExecutionsApi()

    def tearDown(self) -> None:
        pass

    def test_create_execution(self) -> None:
        """Test case for create_execution

        Create a new execution for a flow
        """
        pass

    def test_delete_execution(self) -> None:
        """Test case for delete_execution

        Delete an execution
        """
        pass

    def test_delete_executions_by_ids(self) -> None:
        """Test case for delete_executions_by_ids

        Delete a list of executions
        """
        pass

    def test_delete_executions_by_query(self) -> None:
        """Test case for delete_executions_by_query

        Delete executions filter by query parameters
        """
        pass

    def test_download_file_from_execution(self) -> None:
        """Test case for download_file_from_execution

        Download file for an execution
        """
        pass

    def test_execution(self) -> None:
        """Test case for execution

        Get an execution
        """
        pass

    def test_execution_flow_graph(self) -> None:
        """Test case for execution_flow_graph

        Generate a graph for an execution
        """
        pass

    def test_file_metadatas_from_execution(self) -> None:
        """Test case for file_metadatas_from_execution

        Get file meta information for an execution
        """
        pass

    def test_flow_from_execution(self) -> None:
        """Test case for flow_from_execution

        Get flow information's for an execution
        """
        pass

    def test_flow_from_execution_by_id(self) -> None:
        """Test case for flow_from_execution_by_id

        Get flow information's for an execution
        """
        pass

    def test_follow_dependencies_executions(self) -> None:
        """Test case for follow_dependencies_executions

        Follow all execution dependencies executions
        """
        pass

    def test_follow_execution(self) -> None:
        """Test case for follow_execution

        Follow an execution
        """
        pass

    def test_force_run_by_ids(self) -> None:
        """Test case for force_run_by_ids

        Force run a list of executions
        """
        pass

    def test_force_run_execution(self) -> None:
        """Test case for force_run_execution

        Force run an execution
        """
        pass

    def test_force_run_executions_by_query(self) -> None:
        """Test case for force_run_executions_by_query

        Force run executions filter by query parameters
        """
        pass

    def test_kill_execution(self) -> None:
        """Test case for kill_execution

        Kill an execution
        """
        pass

    def test_kill_executions_by_ids(self) -> None:
        """Test case for kill_executions_by_ids

        Kill a list of executions
        """
        pass

    def test_kill_executions_by_query(self) -> None:
        """Test case for kill_executions_by_query

        Kill executions filter by query parameters
        """
        pass

    def test_latest_executions(self) -> None:
        """Test case for latest_executions

        Get the latest execution for given flows
        """
        pass

    def test_pause_execution(self) -> None:
        """Test case for pause_execution

        Pause a running execution.
        """
        pass

    def test_pause_executions_by_ids(self) -> None:
        """Test case for pause_executions_by_ids

        Pause a list of running executions
        """
        pass

    def test_pause_executions_by_query(self) -> None:
        """Test case for pause_executions_by_query

        Pause executions filter by query parameters
        """
        pass

    def test_replay_execution(self) -> None:
        """Test case for replay_execution

        Create a new execution from an old one and start it from a specified task run id
        """
        pass

    def test_replay_execution_withinputs(self) -> None:
        """Test case for replay_execution_withinputs

        Create a new execution from an old one and start it from a specified task run id
        """
        pass

    def test_replay_executions_by_ids(self) -> None:
        """Test case for replay_executions_by_ids

        Create new executions from old ones. Keep the flow revision
        """
        pass

    def test_replay_executions_by_query(self) -> None:
        """Test case for replay_executions_by_query

        Create new executions from old ones filter by query parameters. Keep the flow revision
        """
        pass

    def test_restart_execution(self) -> None:
        """Test case for restart_execution

        Restart a new execution from an old one
        """
        pass

    def test_restart_executions_by_ids(self) -> None:
        """Test case for restart_executions_by_ids

        Restart a list of executions
        """
        pass

    def test_restart_executions_by_query(self) -> None:
        """Test case for restart_executions_by_query

        Restart executions filter by query parameters
        """
        pass

    def test_resume_execution(self) -> None:
        """Test case for resume_execution

        Resume a paused execution.
        """
        pass

    def test_resume_executions_by_ids(self) -> None:
        """Test case for resume_executions_by_ids

        Resume a list of paused executions
        """
        pass

    def test_resume_executions_by_query(self) -> None:
        """Test case for resume_executions_by_query

        Resume executions filter by query parameters
        """
        pass

    def test_search_concurrency_limits(self) -> None:
        """Test case for search_concurrency_limits

        Search for flow concurrency limits
        """
        pass

    def test_search_executions(self) -> None:
        """Test case for search_executions

        Search for executions
        """
        pass

    def test_search_executions_by_flow_id(self) -> None:
        """Test case for search_executions_by_flow_id

        Search for executions for a flow
        """
        pass

    def test_set_labels_on_terminated_execution(self) -> None:
        """Test case for set_labels_on_terminated_execution

        Add or update labels of a terminated execution
        """
        pass

    def test_set_labels_on_terminated_executions_by_ids(self) -> None:
        """Test case for set_labels_on_terminated_executions_by_ids

        Set labels on a list of executions
        """
        pass

    def test_set_labels_on_terminated_executions_by_query(self) -> None:
        """Test case for set_labels_on_terminated_executions_by_query

        Set label on executions filter by query parameters
        """
        pass

    def test_trigger_execution_by_get_webhook(self) -> None:
        """Test case for trigger_execution_by_get_webhook

        Trigger a new execution by GET webhook trigger
        """
        pass

    def test_unqueue_execution(self) -> None:
        """Test case for unqueue_execution

        Unqueue an execution
        """
        pass

    def test_unqueue_executions_by_ids(self) -> None:
        """Test case for unqueue_executions_by_ids

        Unqueue a list of executions
        """
        pass

    def test_unqueue_executions_by_query(self) -> None:
        """Test case for unqueue_executions_by_query

        Unqueue executions filter by query parameters
        """
        pass

    def test_update_concurrency_limit(self) -> None:
        """Test case for update_concurrency_limit

        Update a flow concurrency limit
        """
        pass

    def test_update_execution_status(self) -> None:
        """Test case for update_execution_status

        Change the state of an execution
        """
        pass

    def test_update_executions_status_by_ids(self) -> None:
        """Test case for update_executions_status_by_ids

        Change executions state by id
        """
        pass

    def test_update_executions_status_by_query(self) -> None:
        """Test case for update_executions_status_by_query

        Change executions state by query parameters
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class QueryFilterField(str, Enum):
    """
    QueryFilterField
    """

    """
    allowed enum values
    """
    QUERY = 'QUERY'
    SCOPE = 'SCOPE'
    NAMESPACE = 'NAMESPACE'
    KIND = 'KIND'
    LABELS = 'LABELS'
    METADATA = 'METADATA'
    FLOW_ID = 'FLOW_ID'
    FLOW_REVISION = 'FLOW_REVISION'
    ID = 'ID'
    ASSET_ID = 'ASSET_ID'
    TYPE = 'TYPE'
    CREATED = 'CREATED'
    UPDATED = 'UPDATED'
    START_DATE = 'START_DATE'
    END_DATE = 'END_DATE'
    STATE = 'STATE'
    TIME_RANGE = 'TIME_RANGE'
    TRIGGER_EXECUTION_ID = 'TRIGGER_EXECUTION_ID'
    TRIGGER_ID = 'TRIGGER_ID'
    TRIGGER_STATE = 'TRIGGER_STATE'
    EXECUTION_ID = 'EXECUTION_ID'
    TASK_ID = 'TASK_ID'
    TASK_RUN_ID = 'TASK_RUN_ID'
    CHILD_FILTER = 'CHILD_FILTER'
    WORKER_ID = 'WORKER_ID'
    EXISTING_ONLY = 'EXISTING_ONLY'
    MIN_LEVEL = 'MIN_LEVEL'
    PATH = 'PATH'
    PARENT_PATH = 'PARENT_PATH'
    VERSION = 'VERSION'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of QueryFilterField from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import regex as re
import json

from pydantic import BaseModel, ConfigDict
from typing import Any, ClassVar, Dict, List, Optional
from kestrapy.models.execution_usage import ExecutionUsage
from kestrapy.models.flow_usage import FlowUsage
from kestrapy.models.group_usage import GroupUsage
from kestrapy.models.role_usage import RoleUsage
from kestrapy.models.tenant_usage import TenantUsage
from kestrapy.models.user_usage import UserUsage
from typing import Optional, Set
from typing_extensions import Self

class UsageEE(BaseModel):
    """
    UsageEE
    """ # noqa: E501
    flows: Optional[FlowUsage] = None
    executions: Optional[ExecutionUsage] = None
    groups: Optional[GroupUsage] = None
    users: Optional[UserUsage] = None
    roles: Optional[RoleUsage] = None
    tenants: Optional[TenantUsage] = None
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["flows", "executions", "groups", "users", "roles", "tenants"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of UsageEE from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of flows
        if self.flows:
            _dict['flows'] = self.flows.to_dict()
        # override the default output from pydantic by calling `to_dict()` of executions
        if self.executions:
            _dict['executions'] = self.executions.to_dict()
        # override the default output from pydantic by calling `to_dict()` of groups
        if self.groups:
            _dict['groups'] = self.groups.to_dict()
        # override the default output from pydantic by calling `to_dict()` of users
        if self.users:
            _dict['users'] = self.users.to_dict()
        # override the default output from pydantic by calling `to_dict()` of roles
        if self.roles:
            _dict['roles'] = self.roles.to_dict()
        # override the default output from pydantic by calling `to_dict()` of tenants
        if self.tenants:
            _dict['tenants'] = self.tenants.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of UsageEE from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "flows": FlowUsage.from_dict(obj["flows"]) if obj.get("flows") is not None else None,
            "executions": ExecutionUsage.from_dict(obj["executions"]) if obj.get("executions") is not None else None,
            "groups": GroupUsage.from_dict(obj["groups"]) if obj.get("groups") is not None else None,
            "users": UserUsage.from_dict(obj["users"]) if obj.get("users") is not None else None,
            "roles": RoleUsage.from_dict(obj["roles"]) if obj.get("roles") is not None else None,
            "tenants": TenantUsage.from_dict(obj["tenants"]) if obj.get("tenants") is not None else None
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


